"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.signWindows = signWindows;
exports.getPSCmd = getPSCmd;
const builder_util_1 = require("builder-util");
async function signWindows(options, packager) {
    const packageManager = await packager.signingManager.value;
    if (options.options.azureSignOptions) {
        if (options.options.signtoolOptions) {
            builder_util_1.log.warn(null, "ignoring signtool options, using Azure Trusted Signing; please only configure one");
        }
        builder_util_1.log.info({ path: builder_util_1.log.filePath(options.path) }, "signing with Azure Trusted Signing (beta)");
    }
    else {
        builder_util_1.log.info({ path: builder_util_1.log.filePath(options.path) }, "signing with signtool.exe");
        const deprecatedFields = {
            sign: options.options.sign,
            signDlls: options.options.signDlls,
            signingHashAlgorithms: options.options.signingHashAlgorithms,
            certificateFile: options.options.certificateFile,
            certificatePassword: options.options.certificatePassword,
            certificateSha1: options.options.certificateSha1,
            certificateSubjectName: options.options.certificateSubjectName,
            additionalCertificateFile: options.options.additionalCertificateFile,
            rfc3161TimeStampServer: options.options.rfc3161TimeStampServer,
            timeStampServer: options.options.timeStampServer,
            publisherName: options.options.publisherName,
        };
        const fields = Object.entries(deprecatedFields)
            .filter(([, value]) => !!value)
            .map(([field]) => field);
        if (fields.length) {
            builder_util_1.log.warn({ fields, reason: "please move to win.signtoolOptions.<field_name>" }, `deprecated field`);
        }
    }
    return signWithRetry(async () => packageManager.signFile(options));
}
function signWithRetry(signer) {
    return (0, builder_util_1.retry)(signer, 3, 1000, 1000, 0, (e) => {
        const message = e.message;
        if (
        // https://github.com/electron-userland/electron-builder/issues/1414
        (message === null || message === void 0 ? void 0 : message.includes("Couldn't resolve host name")) ||
            (
            // https://github.com/electron-userland/electron-builder/issues/8615
            message === null || message === void 0 ? void 0 : message.includes("being used by another process."))) {
            builder_util_1.log.warn({ error: message }, "attempt to sign failed, another attempt will be made");
            return true;
        }
        return false;
    });
}
async function getPSCmd(vm) {
    return await vm
        .exec("powershell.exe", ["-NoProfile", "-NonInteractive", "-Command", `Get-Command pwsh.exe`])
        .then(() => {
        builder_util_1.log.debug(null, "identified pwsh.exe for executing code signing");
        return "pwsh.exe";
    })
        .catch(() => {
        builder_util_1.log.debug(null, "unable to find pwsh.exe, falling back to powershell.exe");
        return "powershell.exe";
    });
}
//# sourceMappingURL=windowsCodeSign.js.map